<?php

declare(strict_types=1);

namespace app\models;

use Yii;

use app\models\Account;
use app\models\Product;
use app\models\SupplyEdgeProduct;
use app\models\traits\Xml2Array;

use carono\exchange1c\interfaces\ProductInterface;

use mirzaev\yii2\arangodb\Query;

/**
 * Поставка (выгрузка товаров от поставщиков)
 *
 * Представляет собой предложения от поставщиков которые добавляются
 * в универсальные лоты товаров в асспортименте магазина
 *
 * @see Product Продукт (туда добавляются поставки)
 */
class Supply extends Product implements ProductInterface
{
    /**
     * Метод для конвертации XML в Array
     */
    use Xml2Array;

    /**
     * Имя коллекции
     */
    public static function collectionName(): string
    {
        return 'supply';
    }

    /**
     * Свойства
     */
    public function attributes(): array
    {
        return array_merge(
            parent::attributes(),
            [
                'onec'
            ]
        );
    }

    /**
     * Метки свойств
     */
    public function attributeLabels(): array
    {
        return array_merge(
            parent::attributeLabels(),
            [
                'onec' => 'Данные 1С'
            ]
        );
    }

    /**
     * После сохранения
     */
    public function afterSave($data, $vars): void
    {
        // Запись ребра: АККАУНТ -> ПОСТАВКА
        (new AccountEdgeSupply)->write(Yii::$app->user->id, $this->readId(), 'import');
    }

    /**
     * Запись реквизитов из 1С
     */
    public function setRequisite1c($name, $value): mixed
    {
        return true;
    }

    /**
     * Запись группы из 1С
     */
    public function setGroup1c($group): mixed
    {
        // Чтение группы
        // if ($group = SupplyGroup::readByOnecId($group->id)) {
        //     // Запись ребра: ПОСТАВКА => ГРУППА ПОСТАВОК
        //     return static::writeEdgeBetweenGroup(static::collectionName() . '/' . $this->_key, $group->collectionName() . '/' . $group->_key);
        // }

        return true;
    }

    /**
     * Запись данных свойств по UUID 1C
     *
     * Ищет записанные свойства из 1C по их идентификатору и добавляет к ним
     * недостающие данные. Это костыль оставшийся от реляционных баз данных
     */
    public static function createProperties1c($properties): void
    {
        // Инициализация
        $models = static::searchOnecByAccountId(Yii::$app->user->id, true);
        $properties = self::xml2array($properties->xml);

        foreach ($models as $model) {
            // Перебор записей

            // Инициализация
            $changes = false;

            foreach ($model->onec['ЗначенияСвойств'] as &$attribute) {
                // Перебор аттрибутов

                foreach ($properties as $property) {
                    // Перебор свойств

                    if ($attribute['ЗначенияСвойства']['Ид'] === $property['Свойство']['Ид']) {
                        // Совпадение идентификаторов

                        // Объединение данных
                        array_merge($attribute, $property);

                        // Запись индикатора наличия изменений
                        $changes = true;
                    }
                }
            }

            if ($changes) {
                $model->save();
            }
        }
    }

    /**
     * Запись параметров из 1С
     */
    public function setProperty1c($property): mixed
    {
        return true;
    }

    /**
     * Запись изображений из 1С
     */
    public function addImage1c($path, $caption): mixed
    {
        return true;
    }

    /**
     * Запись ребра (предложения от поставок к продуктам) из 1С
     */
    public function getOffer1c($offer): SupplyEdgeProduct
    {
        if (empty($this->catn)) {
            // Не передан каталожный номер

            // Разработчику библеотеки надо дать по жопе
            return new SupplyEdgeProduct;
        } else if (!$product = Product::searchByCatn($this->catn)) {
            // Продукт не найден

            if (!$this->initProduct()) {
                // Не удалось инициализировать продукт

                // Разработчику библеотеки надо дать по жопе
                return new SupplyEdgeProduct;
            }
        }

        $product = Product::searchByCatn($this->catn);

        // Запись ребра: ПОСТАВКА -> ПРОДУКТ
        return (new SupplyEdgeProduct)->write(
            $this->readId(),
            $product->readId(),
            'sell',
            [
                'onec' => self::xml2array($offer->xml)
            ]
        );
    }

    /**
     * Запись продукта из 1С
     */
    public static function createModel1c($product): ?self
    {
        // Инициализация
        $model = self::searchByOnecId($id = (string) $product->Ид) ?? new self;

        // Настройка
        $model->ocid = $id ?? null;
        $model->catn = (string) $product->Артикул;
        $model->oemn = null;
        $model->onec = self::xml2array($product->xml);

        // Запись
        return $model->save() ? $model : null;
    }

    /**
     * Инициализация продукта
     */
    protected function initProduct(): ?Product
    {
        // Надо не забыть сделать выборку полей и ручное подключение

        if (empty($this->catn)) {
            // Не передан каталожный номер

            return false;
        } else if (Product::searchByCatn($this->catn)) {
            // Продукт уже был инициализирован

            return true;
        }

        // Инициализация
        $product = new Product();

        // Настройки
        $product->catn = $this->catn;

        // Запись
        return $product->save() ? $product : null;
    }

    /**
     * Запись цены из 1С
     */
    public function setPrice1c($price): mixed
    {
        return true;
    }

    /**
     * Запись данных на случай ошибки при экспорте из 1С
     */
    public function setRaw1cData($cml, $object): bool
    {
        return true;
    }

    /**
     * Поиск по идентификатору из 1С
     *
     * @param string $ocid Идентификатор из 1С
     *
     * @return Supply|null
     */
    public static function searchByOnecId(string $ocid): ?Supply
    {
        return static::findOne([static::getIdFieldName1c() => $ocid]);
    }

    /**
     * Чтение группы из 1С
     */
    public function getGroup1c(): ?SupplyGroup
    {
        return $this->group;
    }

    /**
     * Чтение названия поля в котором хранится идентификатор из 1С
     */
    public static function getIdFieldName1c(): string
    {
        return 'ocid';
    }

    public static function searchOnecByAccountId(string $id, bool $full = false): array
    {
        $subquery = static::find()
            ->for(['account', 'account_edge_supply'])
            ->traversal('supply')->in('account_edge_supply')
            ->where(['account._id' => $id])
            ->select('account_edge_supply')
            ->createCommand();

        $query = static::find()
            ->addParams($subquery->getBindVars())
            ->let('account_edge_supply', '(' . (string) $subquery . ')')
            ->where('supply._id == account_edge_supply[0]._to')
            ->andWhere('supply.onec["ЗначенияСвойств"] != null');

        if ($full) {
            // Если указан полный поиск
            return $query->select('supply')->all();
        }

        $query = $query->select('supply.onec["ЗначенияСвойств"]')->createCommand()->execute()->getAll();

        foreach ($query as &$attribute) {
            $attribute = $attribute->getAll();
        }

        return $query;
    }
}
